/****************************************************************************
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the QtCore module of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Licensees holding valid Qt Enterprise licenses may use this file in
** accordance with the Qt Enterprise License Agreement provided with the 
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  
**
** If you have questions regarding the use of this file, please use
** contact form at http://qt.digia.com
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QLOCALE_BLACKBERRY_H
#define QLOCALE_BLACKBERRY_H

#include "qsocketnotifier.h"
#include "qreadwritelock.h"
#include "qlocale.h"

QT_BEGIN_HEADER

QT_BEGIN_NAMESPACE

QT_MODULE(Core)

#ifndef QT_NO_SYSTEMLOCALE

class QBBSystemLocaleData : public QObject
{
    Q_OBJECT

public:
    QBBSystemLocaleData();
    virtual ~QBBSystemLocaleData();
    uint measurementSystem();
    QVariant timeFormat(QLocale::FormatType);
    QVariant dateTimeFormat(QLocale::FormatType);
    QLocale languageLocale();
    QLocale regionLocale();

    QReadWriteLock lock;

public Q_SLOTS:
    void installSocketNotifiers();
    void readLangageLocale();
    void readRegionLocale();
    void readMeasurementSystem();
    void readHourFormat();

private:
    QByteArray readPpsValue(const char *ppsObject, int ppsFd);
    QString getCorrectFormat(const QString &baseFormat, QLocale::FormatType typeFormat);

    QByteArray lc_langage;
    QByteArray lc_region;
    uint m_measurementSystem;
    bool is24HourFormat;

    QSocketNotifier *languageNotifier;
    QSocketNotifier *regionNotifier;
    QSocketNotifier *measurementNotifier;
    QSocketNotifier *hourNotifier;

    int languageFd;
    int regionFd;
    int measurementFd;
    int hourFd;
};
#endif // QT_NO_SYSTEMLOCALE

QT_END_NAMESPACE

QT_END_HEADER

#endif // QLOCALE_BLACKBERRY_H

